==> homeassistant-2024.4.0-py3-none-any.whl: homeassistant/components/ring/switch.py <==
"""Component providing HA switch support for Ring Door Bell/Chimes."""

from datetime import timedelta
import logging
from typing import Any

from ring_doorbell import RingGeneric, RingStickUpCam

from homeassistant.components.switch import SwitchEntity
from homeassistant.config_entries import ConfigEntry
from homeassistant.core import HomeAssistant, callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback
import homeassistant.util.dt as dt_util

from .const import DOMAIN, RING_DEVICES, RING_DEVICES_COORDINATOR
from .coordinator import RingDataCoordinator
from .entity import RingEntity, exception_wrap

_LOGGER = logging.getLogger(__name__)


# It takes a few seconds for the API to correctly return an update indicating
# that the changes have been made. Once we request a change (i.e. a light
# being turned on) we simply wait for this time delta before we allow
# updates to take place.

SKIP_UPDATES_DELAY = timedelta(seconds=5)


async def async_setup_entry(
    hass: HomeAssistant,
    config_entry: ConfigEntry,
    async_add_entities: AddEntitiesCallback,
) -> None:
    """Create the switches for the Ring devices."""
    devices = hass.data[DOMAIN][config_entry.entry_id][RING_DEVICES]
    coordinator: RingDataCoordinator = hass.data[DOMAIN][config_entry.entry_id][
        RING_DEVICES_COORDINATOR
    ]

    async_add_entities(
        SirenSwitch(device, coordinator)
        for device in devices["stickup_cams"]
        if device.has_capability("siren")
    )


class BaseRingSwitch(RingEntity, SwitchEntity):
    """Represents a switch for controlling an aspect of a ring device."""

    def __init__(
        self, device: RingGeneric, coordinator: RingDataCoordinator, device_type: str
    ) -> None:
        """Initialize the switch."""
        super().__init__(device, coordinator)
        self._device_type = device_type
        self._attr_unique_id = f"{self._device.id}-{self._device_type}"


class SirenSwitch(BaseRingSwitch):
    """Creates a switch to turn the ring cameras siren on and off."""

    _attr_translation_key = "siren"

    def __init__(self, device: RingGeneric, coordinator: RingDataCoordinator) -> None:
        """Initialize the switch for a device with a siren."""
        super().__init__(device, coordinator, "siren")
        self._no_updates_until = dt_util.utcnow()
        self._attr_is_on = device.siren > 0

    @callback
    def _handle_coordinator_update(self):
        """Call update method."""
        if self._no_updates_until > dt_util.utcnow():
            return

        if (device := self._get_coordinator_device()) and isinstance(
            device, RingStickUpCam
        ):
            self._attr_is_on = device.siren > 0
        super()._handle_coordinator_update()

    @exception_wrap
    def _set_switch(self, new_state):
        """Update switch state, and causes Home Assistant to correctly update."""
        self._device.siren = new_state

        self._attr_is_on = new_state > 0
        self._no_updates_until = dt_util.utcnow() + SKIP_UPDATES_DELAY
        self.schedule_update_ha_state()

    def turn_on(self, **kwargs: Any) -> None:
        """Turn the siren on for 30 seconds."""
        self._set_switch(1)

    def turn_off(self, **kwargs: Any) -> None:
        """Turn the siren off."""
        self._set_switch(0)

==> homeassistant-2024.8.0b1-py3-none-any.whl: homeassistant/components/ring/switch.py <==
"""Component providing HA switch support for Ring Door Bell/Chimes."""

from datetime import timedelta
import logging
from typing import Any

from ring_doorbell import RingStickUpCam

from homeassistant.components.switch import SwitchEntity
from homeassistant.config_entries import ConfigEntry
from homeassistant.core import HomeAssistant, callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback
import homeassistant.util.dt as dt_util

from . import RingData
from .const import DOMAIN
from .coordinator import RingDataCoordinator
from .entity import RingEntity, exception_wrap

_LOGGER = logging.getLogger(__name__)


# It takes a few seconds for the API to correctly return an update indicating
# that the changes have been made. Once we request a change (i.e. a light
# being turned on) we simply wait for this time delta before we allow
# updates to take place.

SKIP_UPDATES_DELAY = timedelta(seconds=5)


async def async_setup_entry(
    hass: HomeAssistant,
    config_entry: ConfigEntry,
    async_add_entities: AddEntitiesCallback,
) -> None:
    """Create the switches for the Ring devices."""
    ring_data: RingData = hass.data[DOMAIN][config_entry.entry_id]
    devices_coordinator = ring_data.devices_coordinator

    async_add_entities(
        SirenSwitch(device, devices_coordinator)
        for device in ring_data.devices.stickup_cams
        if device.has_capability("siren")
    )


class BaseRingSwitch(RingEntity[RingStickUpCam], SwitchEntity):
    """Represents a switch for controlling an aspect of a ring device."""

    def __init__(
        self, device: RingStickUpCam, coordinator: RingDataCoordinator, device_type: str
    ) -> None:
        """Initialize the switch."""
        super().__init__(device, coordinator)
        self._device_type = device_type
        self._attr_unique_id = f"{self._device.id}-{self._device_type}"


class SirenSwitch(BaseRingSwitch):
    """Creates a switch to turn the ring cameras siren on and off."""

    _attr_translation_key = "siren"

    def __init__(
        self, device: RingStickUpCam, coordinator: RingDataCoordinator
    ) -> None:
        """Initialize the switch for a device with a siren."""
        super().__init__(device, coordinator, "siren")
        self._no_updates_until = dt_util.utcnow()
        self._attr_is_on = device.siren > 0

    @callback
    def _handle_coordinator_update(self) -> None:
        """Call update method."""
        if self._no_updates_until > dt_util.utcnow():
            return
        device = self._get_coordinator_data().get_stickup_cam(
            self._device.device_api_id
        )
        self._attr_is_on = device.siren > 0
        super()._handle_coordinator_update()

    @exception_wrap
    def _set_switch(self, new_state: int) -> None:
        """Update switch state, and causes Home Assistant to correctly update."""
        self._device.siren = new_state

        self._attr_is_on = new_state > 0
        self._no_updates_until = dt_util.utcnow() + SKIP_UPDATES_DELAY
        self.schedule_update_ha_state()

    def turn_on(self, **kwargs: Any) -> None:
        """Turn the siren on for 30 seconds."""
        self._set_switch(1)

    def turn_off(self, **kwargs: Any) -> None:
        """Turn the siren off."""
        self._set_switch(0)
